###global _:true,expect:true,###
describe "_async", ->

  beforeEach ->

    @injects.push (uiGmap_async) =>
      @subject = uiGmap_async

    @injectAll()

  afterEach ->
    @subject = null

  describe 'getArrayAndKeys', ->
    describe 'handles array', ->
      it 'normal', (done) ->
        testColl = [1,2,3]
        @subject.getArrayAndKeys testColl, null, null, (array, keys) ->
          expect(testColl).toEqual array
          expect(keys).toBeFalsy()
          done()

    describe 'handles object', ->
      it 'normal', (done) ->
        testColl =
          'one':1
          'two':2
          'three':3
        @subject.getArrayAndKeys testColl, null, (-> throw "bail"), (array, keys) ->
          expect(['one','two','three']).toEqual array
          expect(keys).toBeFalsy()
          done()

      it 'came w keys', (done) ->
        testColl =
          '1':1
          '2':2
          '3':3
        @subject.getArrayAndKeys testColl, ['a','b','c'], (-> throw "bail"), (array, keys) ->
          expect(['a','b','c']).toEqual array
          expect(['a','b','c']).toEqual keys
          done()

  describe "arrays", ->

    it "handle zero elements", (done) ->
      @digest =>
        known = []
        test = []
        pauses = 0

        @subject.each known, (num) ->
          test.push(num)
        , 100
        , ->
          pauses++
        .then ->
          expect(pauses).toEqual(0)
          expect(test.length).toEqual(known.length)
          expect(test).toEqual(known)
          done()

    it "handle array of 101 outputs 101 elements equal to the original, with 1 pauses", (done) ->
      @digest =>
        known = _.range(101)
        test = []
        pauses = 1

        @subject.each known, (num) ->
          test.push(num)
        , 100
        , ->
          pauses++
        .then ->
          expect(pauses).toEqual(2)
          expect(test.length).toEqual(known.length)
          expect(test).toEqual(known)
          done()

    it "handle callback passes an index", (done) ->
      @digest =>
        chunkHit = false
        @subject.each [1], (thing, index) ->
          chunkHit = true
          expect(thing).toEqual 1
          expect(index).toEqual 0
        .then ->
          expect(chunkHit).toBeTruthy()
          done()

    it "handle array of 200 outputs 200 elements equal to the original, with 2 pauses", (done) ->
      @digest =>
        known = _.range(200)
        test = []
        pauses = 1

        @subject.each known, (num) ->
          test.push(num)
        , 100
        , ->
          pauses++
        .then ->
          expect(pauses).toEqual(2)
          expect(test.length).toEqual(known.length)
          expect(test).toEqual(known)
          done()

    it "handle array of 1000 outputs 1000 elements equal to the original, with 10 pauses", (done) ->
      @digest =>
        known = _.range(1000)
        test = []
        pauses = 1
        @subject.each known, (num) ->
          test.push(num)
        , 100
        , ->
          pauses++
        .then ->
          expect(test.length).toEqual(known.length)
          expect(test).toEqual(known)
          expect(pauses).toEqual(10)
          done()
      , 10

    describe 'map', ->
      it "handle 1000 outputs 1000 elements equal to the original, with 10 pauses", (done) ->
        @digest =>
          known = _.range(1000)
          test = []
          pauses = 1
          @subject.map known, (num) ->
            num += 1
            "$#{num.toString()}"
          , 100
          , ->
            pauses++
          .then (mapped) ->
            test = mapped
            expect(test[999]).toEqual("$1000")
            expect(test.length).toEqual(known.length)
            expect(test).toEqual(
              _.map known, (n) ->
                n += 1
                "$#{n.toString()}"
            )
            expect(pauses).toEqual(10)
            done()
        , 10

    describe "no chunking / pauses", ->
      it "range 101 zero pauses", (done) ->
        @digest =>
          known = _.range(101)
          test = []
          pauses = 0
          @subject.each(known, (num) ->
            test.push(num)
          , chunking = false
          , ->
            pauses++
          ).then ->
            expect(pauses).toEqual(0) #it should not be hit
            expect(test.length).toEqual(known.length)
            expect(test).toEqual(known)
            done()

      it 'prototypes should not be called', (done) ->
        class Dummy
          prop1: 'prop1'
          prop2: 'prop2'
        d = new Dummy()
        d.prop3 = 'prop3'

        vals = []

        @digest =>
          @subject.each d, (val) ->
            vals.push val
          , chunking = false
          .then ->
            expect(vals.length).toBe 1
            expect(vals[0]).toBe d.prop3
            done()


  describe "Objects", ->
    it "handle zero elements", (done) ->
      @digest =>
        known = {}
        test = []
        pauses = 0

        @subject.each known, (num) ->
          test.push(num)
        , 100
        , ->
          pauses++
        .then ->
          expect(pauses).toEqual(0)
          vals = _.values(_.omit known, 'length')
          expect(test.length).toEqual(vals.length)
          expect(test).toEqual(vals)
          done()

    it "101 outputs 101 elements equal to the original, with 1 pauses", (done) ->
      @digest =>
        known = {}
        known[num] = num for num in [0..100]
        # known.length = _.values(known).length
        test = []
        pauses = 1

        @subject.each known, (num) ->
          test.push(num)
        , 100
        , ->
          pauses++
        .then ->
          expect(pauses).toEqual(2)
          vals = _.values(_.omit known, 'length')
          expect(test.length).toEqual(vals.length)
          expect(test).toEqual(vals)
          done()

    it "handle 1000 outputs 1000 elements equal to the original, with 10 pauses", (done) ->
      @digest =>
        known = {}
        known[num] = num for num in [0..999]
        known.length = _.values(known).length
        test = []
        pauses = 1
        @subject.each known, (num) ->
          test.push(num)
        , 100
        , ->
          pauses++
        .then ->
          vals = _.values(_.omit known, 'length')
          expect(test.length).toEqual(vals.length)
          expect(test).toEqual(vals)
          expect(pauses).toEqual(10)
          done()
      , 10

    describe 'map', ->
      it "handle 1000 outputs 1000 elements equal to the original, with 10 pauses", (done) ->
        @digest =>
          known = {}
          known[num] = num for num in [0..999]
          test = []
          pauses = 1
          @subject.map known, (num) ->
            num += 1
            "$#{num.toString()}"
          , 100
          , ->
            pauses++
          .then (mapped) ->
            test = mapped
            expect(test[999]).toEqual("$1000")
            vals = _.values(_.omit known, 'length')
            expect(test.length).toEqual(vals.length)
            expect(test).toEqual(
              _.map vals, (n) ->
                n += 1
                "$#{n.toString()}"
            )
            expect(pauses).toEqual(10)
            done()
        , 10

  describe 'chunkSizeFrom', ->
    it 'undefined returns undefined', ->
      expect(@subject.chunkSizeFrom(undefined)).toBeFalsy()
      #note this allows chunkSize to default to a chunk of 20 or some chunk
      #if it returns false then chunking is disabled by default on plurals
      expect(@subject.chunkSizeFrom(undefined) == undefined).toBeTruthy()

    it 'false returns false', ->
      expect(@subject.chunkSizeFrom(false)).toBeFalsy()
      expect(@subject.chunkSizeFrom(false) == false).toBeTruthy()

    it 'NO returns false', ->
      expect(@subject.chunkSizeFrom('NO')).toBeFalsy()
      expect(@subject.chunkSizeFrom('NO') == false).toBeTruthy()

    it 'FALSE returns false', ->
      expect(@subject.chunkSizeFrom('FALSE')).toBeFalsy()
      expect(@subject.chunkSizeFrom('FALSE') == false).toBeTruthy()

    it 'number returns number', ->
      expect(@subject.chunkSizeFrom(300)).toBe(300)

    it 'string number returns undefined', ->
      expect(@subject.chunkSizeFrom('300')).toBeUndefined()

    it 'non number returns undefined', ->
      expect(@subject.chunkSizeFrom('3-00')).toBeUndefined()
